/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.generator.lifetime;

import java.util.Random;

import de.tudarmstadt.maki.simonstrator.api.Randoms;
import de.tudarmstadt.maki.simonstrator.api.Time;
import de.tudarmstadt.maki.simonstrator.api.util.XMLConfigurableConstructor;

public class ExponentialLifetimeDistribution implements LifetimeDistribution {
	private Random eventDurationRandom = Randoms.getRandom(getClass());

	private long _averageEventDuration;
	private double _existProbability;

	private long _granularity = 10 * Time.SECOND;

	@XMLConfigurableConstructor({ "duration" })
	public ExponentialLifetimeDistribution(long pEventDuration) {
		_averageEventDuration = pEventDuration;

		_existProbability = 1 - 1 / (double) (pEventDuration / _granularity);
	}

	@Override
	public long getLifetime() {
		double random = eventDurationRandom.nextDouble();

		long theoreticLifetime = (long) Math.ceil(Math.log(random) / Math.log(_existProbability));
		long lifetime = Math.max(theoreticLifetime * _granularity, _granularity * 2);
		System.out.println("Random lifetime: " + random + " with duration: "
				+ theoreticLifetime * _granularity + " (final: " + lifetime + ")");

		return lifetime;
	}

	@Override
	public long getAverageLifetime() {
		return _averageEventDuration;
	}
}
