/*
 * Copyright (c) 2005-2018 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


package de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.plugin;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import de.tudarmstadt.maki.simonstrator.api.Host;
import de.tudarmstadt.maki.simonstrator.api.Randoms;
import de.tudarmstadt.maki.simonstrator.api.component.ComponentNotAvailableException;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.EnvironmentProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.FogProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.RoadProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSComponent;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSRequest;
import de.tudarmstadt.maki.simonstrator.api.component.sis.exception.InformationNotAvailableException;
import de.tudarmstadt.maki.simonstrator.api.component.sis.type.SiSTypes;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkEdge;
import de.tudarmstadt.maki.simonstrator.api.util.XMLConfigurableConstructor;

/**
 * @author Bjoern Bueschke
 * @version 1.0 at 27.03.2018
 *
 */
public class FogEnvironmentSensorPlugin implements EnvironmentSensorPlugin {
	private Host _host;

	private SiSComponent _sis;

	private double _accuracy;

	private static Random _random = Randoms.getRandom(FogEnvironmentSensorPlugin.class);

	@XMLConfigurableConstructor({ "accuracy" })
	public FogEnvironmentSensorPlugin(double pAccuracy) {
		setAccuracy(pAccuracy);
	}

	@Override
	public void setHost(Host pHost) {
		_host = pHost;
	}

	@Override
	public List<EnvironmentProperty> getEnvironmentProperties() {
		if (_sis == null) {
			try {
				_sis = _host.getComponent(SiSComponent.class);
			} catch (ComponentNotAvailableException e) {
				throw new AssertionError("SiS requried!", e);
			}
		}
		
		try {
			RoadNetworkEdge edge = _sis.get().localState(SiSTypes.ROAD_EDGE, SiSRequest.NONE);
			Location location = _sis.get().localState(SiSTypes.PHY_LOCATION, SiSRequest.NONE);
			boolean fog = false;
			for (RoadProperty roadProperty : edge.getActiveProperties()) {
				if (roadProperty instanceof FogProperty) {
					if (((FogProperty) roadProperty).hasFog()) {
						fog = true;
					}
				}
			}
			if (_random.nextDouble() >= _accuracy) {
				fog = !fog;
			}
			List<EnvironmentProperty> properties = new ArrayList<>();
			EnvironmentProperty property = new FogProperty(location, edge, fog);
			properties.add(property);
			return properties;
		} catch (InformationNotAvailableException e) {
			throw new AssertionError(SiSTypes.ROAD_EDGE + " and " + SiSTypes.PHY_LOCATION + " are required!");
		}
	}

	@Override
	public Class<? extends RoadProperty> getSupportedClass() {
		return FogProperty.class;
	}

	@Override
	public FogEnvironmentSensorPlugin clone() throws CloneNotSupportedException {
		return new FogEnvironmentSensorPlugin(_accuracy);
	}

	public void setAccuracy(double pAccuracy) {
		_accuracy = pAccuracy;
	}

}
