/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.vehicular.information;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.tudarmstadt.maki.simonstrator.api.Time;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkEdge;

public class DefaultVehicularPointInformation<S extends Object> implements VehicularPointInformation<S> {

	private Location _location;

	private long _detectionTime;

	private S _value;

	private Map<AvailableInformationAttributes, Object> _attributes = new HashMap<>();

	public DefaultVehicularPointInformation(RoadNetworkEdge pEdge, Location pLocation, long pDetectionTime, S pValue) {
		_location = pLocation;
		_detectionTime = pDetectionTime;

		_value = pValue;

		setAttribute(AvailableInformationAttributes.EDGE, pEdge);
	}

	@Override
	public Location getLocation() {
		return _location;
	}

	@Override
	public long getDetectionDate() {
		return _detectionTime;
	}

	@Override
	public RoadNetworkEdge getEdge() {
		return getAttribute(AvailableInformationAttributes.EDGE);
	}

	@Override
	public S getValue() {
		return _value;
	}

	@Override
	public <T> void setAttribute(AvailableInformationAttributes pKey, T pValue) {
		_attributes.put(pKey, pValue);
	}

	@Override
	public <T> T getAttribute(AvailableInformationAttributes pKey) {
		if (_attributes.containsKey(pKey)) {
			return (T) _attributes.get(pKey);
		}
		return null;
	}

	@Override
	public List<AvailableInformationAttributes> getAvailableAttributes() {
		return new ArrayList<>(_attributes.keySet());
	}

	@Override
	public <T> boolean hasAttribute(AvailableInformationAttributes pKey) {
		return _attributes.containsKey(pKey);
	}

	@Override
	public boolean isValid() {
		return getDetectionDate() + (long) getAttribute(AvailableInformationAttributes.TTL) > Time.getCurrentTime();
	}

}
