package de.tudarmstadt.maki.simonstrator.api.web;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

import de.tudarmstadt.maki.simonstrator.api.web.options.WebRunnerOptions;

@XmlRootElement(name = "configuration")
public class WebConfiguration {
    @XmlElementWrapper(name = "scenarios")
    @XmlElement(name = "scenario")
    List<Scenario> _scenarios = new ArrayList<>();

	@XmlElementWrapper(name = "metrics")
	@XmlElement(name = "metric")
	List<Metric> _metrics = new ArrayList<>();

    @XmlElement(name = "options")
    WebRunnerOptions _options = new WebRunnerOptions();

    public Scenario createScenario(String pName, String pConfig) {
        return new Scenario(pName, pConfig);
    }

    public void addScenario(Scenario pScenario) {
        if (getScenario(pScenario.getName()) != null) {
            _scenarios.remove(getScenario(pScenario.getName()));
        }
        _scenarios.add(pScenario);
    }

    public List<Scenario> getScenarios() {
        return _scenarios;
    }

    public WebRunnerOptions getOptions() {
        if (_options == null) {
            _options = new WebRunnerOptions();
        }
        return _options;
    }

    public boolean store(File pScenarioFile) {
        try {
            JAXBContext context = JAXBContext.newInstance(WebConfiguration.class);
            Marshaller marshaller = context.createMarshaller();
            FileOutputStream output = new FileOutputStream(pScenarioFile);
            marshaller.marshal(this, output);
            output.flush();
            output.close();
            return true;
        } catch (JAXBException | IOException e) {
            e.printStackTrace();
            return false;
        }
    }

    public static WebConfiguration load(File pScenarioFile) {
        try {
            JAXBContext context = JAXBContext.newInstance(WebConfiguration.class);
            Unmarshaller unmarshaller = context.createUnmarshaller();
            FileInputStream input = new FileInputStream(pScenarioFile);
            return (WebConfiguration) unmarshaller.unmarshal(input);
        } catch (JAXBException | IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    public Scenario getScenario(String pScenario) {
        for (Scenario scenario : _scenarios) {
            if (scenario.getName().equals(pScenario)) {
                return scenario;
            }
        }
        return null;
    }

	public void removeScenario(Scenario pScenario) {
		_scenarios.remove(pScenario);
	}

	public List<Metric> getMetrics() {
		return _metrics;
	}

	public Metric getMetric(String pMetricName) {
		for (Metric metric : _metrics) {
			if (metric.getName().equals(pMetricName)) {
				return metric;
			}
		}
		return null;
	}

	public void addMetric(Metric pMetric) {
		_metrics.add(pMetric);
	}
}
